"{ Package: 'jn:refactoring_custom' }"

CustomCodeGeneratorTestCase subclass:#CustomAccessMethodsCodeGeneratorTests
	instanceVariableNames:'expectedSource class'
	classVariableNames:''
	poolDictionaries:''
	category:'Interface-Refactoring-Custom-Tests'
!

!CustomAccessMethodsCodeGeneratorTests methodsFor:'accessing'!

generator

    ^ mock mockOf: CustomAccessMethodsCodeGenerator

    "Modified: / 01-07-2014 / 16:19:16 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomAccessMethodsCodeGeneratorTests methodsFor:'tests'!

test_arg_name_for_method_name_boolean
    | actualArgName |

    actualArgName := generator argNameForMethodName: 'isSomething'.

    self assert: 'aBoolean' = actualArgName

    "Created: / 04-07-2014 / 12:38:50 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_arg_name_for_method_name_other
    | actualArgName |

    actualArgName := generator argNameForMethodName: 'selector'.

    self assert: 'something' = actualArgName

    "Created: / 04-07-2014 / 12:39:48 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_build_in_context_with_none_selected_variable
    | class methodCount |

    class := self createClass.
    class instanceVariableNames: 'instanceVariable_01 instanceVariable_02 instanceVariable_03'.

    context selectedClasses: (Array with: class).

    generator mockSelector: #sourceForClass:variableName: withBlockValue: [
        :aClass :varName |

        ^ varName, ' ^ ', varName. 
    ].

    methodCount := class methodDictionary size.

    generator executeInContext: context.

    self assert: (methodCount + 3) = class methodDictionary size.

    expectedSource := 'instanceVariable_02
    ^ instanceVariable_02'.
    self assertMethodSource: expectedSource atSelector: #instanceVariable_02.

    expectedSource := 'instanceVariable_01
    ^ instanceVariable_01'.
    self assertMethodSource: expectedSource atSelector: #instanceVariable_01.

    expectedSource := 'instanceVariable_03
    ^ instanceVariable_03'.
    self assertMethodSource: expectedSource atSelector: #instanceVariable_03.

    "Created: / 20-06-2014 / 20:29:25 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_build_in_context_with_one_selected_variable
    | class methodCount |

    class := self createClass.
    class instanceVariableNames: 'instanceVariable_01 instanceVariable_02 instanceVariable_03'.

    context selectedClasses: (Array with: class).
    context selectedVariables: (Array with: 'instanceVariable_02').

    generator mockSelector: #sourceForClass:variableName: withBlockValue: [
        :aClass :varName |

        ^ varName, ' ^ ', varName. 
    ].

    methodCount := class methodDictionary size.

    generator executeInContext: context.

    self assert: (methodCount + 1) = class methodDictionary size.

    expectedSource := 'instanceVariable_02
    ^ instanceVariable_02'.
    self assertMethodSource: expectedSource atSelector: #instanceVariable_02.

    "Created: / 17-06-2014 / 08:57:28 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified (format): / 20-06-2014 / 20:15:37 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_build_in_context_with_two_selected_variables
    | class methodCount |

    class := self createClass.
    class instanceVariableNames: 'instanceVariable_01 instanceVariable_02 instanceVariable_03'.

    context selectedClasses: (Array with: class).
    context selectedVariables: (Array with: 'instanceVariable_02' with: 'instanceVariable_01').

    generator mockSelector: #sourceForClass:variableName: withBlockValue: [
        :aClass :varName |

        ^ varName, ' ^ ', varName. 
    ].

    methodCount := class methodDictionary size.

    generator executeInContext: context.

    self assert: (methodCount + 2) = class methodDictionary size.

    expectedSource := 'instanceVariable_02
    ^ instanceVariable_02'.
    self assertMethodSource: expectedSource atSelector: #instanceVariable_02.

    expectedSource := 'instanceVariable_01
    ^ instanceVariable_01'.
    self assertMethodSource: expectedSource atSelector: #instanceVariable_01.

    "Created: / 20-06-2014 / 20:26:02 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_default_method_name_for_variable_name
    | actualDefaultMethodName |

    actualDefaultMethodName := generator defaultMethodNameFor: 'varName'.

    self assert: 'defaultVarName' = actualDefaultMethodName

    "Created: / 29-06-2014 / 23:33:34 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_method_name_for_all_upper_case
    | actualMethodName |

    actualMethodName := generator methodNameFor: 'VAR'.

    self assert: 'VAR' = actualMethodName

    "Created: / 20-06-2014 / 21:11:56 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_method_name_for_class_variable
    | actualMethodName |

    actualMethodName := generator methodNameFor: 'ClassVar'.

    self assert: 'classVar' = actualMethodName

    "Created: / 20-06-2014 / 20:36:02 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_method_name_for_instance_variable

    self assert: 'instVar' = (generator methodNameFor: 'instVar')

    "Created: / 20-06-2014 / 20:35:22 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_method_name_for_number
    | actualMethodName |

    actualMethodName := generator methodNameFor: 'Var_7'.

    self assert: 'var_7' = actualMethodName

    "Created: / 20-06-2014 / 21:13:36 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_var_type_of_class_instance_variable
    | actualVarType |

    class := self createClass.
    class
        isMeta: true;
        instanceVariableNames: 'ClassInstVar1 ClassInstVar2'.

    self assert: class isMeta.

    actualVarType := generator varTypeOf: 'ClassInstVar2' class: class.

    self assert: 'classInstVar' = actualVarType

    "Created: / 23-06-2014 / 19:27:17 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_var_type_of_instance_variable
    | actualVarType |

    class := self createClass.
    class instanceVariableNames: 'instVar1 instVar2'.

    actualVarType := generator varTypeOf: 'instVar2' class: class.

    self assert: 'instance' = actualVarType

    "Created: / 23-06-2014 / 19:26:28 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_var_type_of_static
    | actualVarType |

    class := self createClass.
    class classVariableNames: 'ClassVar1 ClassVar2'.

    actualVarType := generator varTypeOf: 'ClassVar2' class: class.

    self assert: 'static' = actualVarType

    "Created: / 20-06-2014 / 21:22:15 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

