"{ Package: 'jn:refactoring_custom' }"

CustomCodeGeneratorOrRefactoring subclass:#CustomCodeGenerator
	instanceVariableNames:'builder dialog changeManager'
	classVariableNames:''
	poolDictionaries:''
	category:'Interface-Refactoring-Custom'
!


!CustomCodeGenerator class methodsFor:'instance creation'!

new
    "return an initialized instance"

    ^ self basicNew initialize.
!

subGeneratorOf: aCodeGenerator
    | codeGenerator nestingCount |

    codeGenerator := self new.
    codeGenerator builder: aCodeGenerator builder.
    codeGenerator userPreferences: aCodeGenerator userPreferences.
    codeGenerator dialog: aCodeGenerator dialog.
    codeGenerator changeManager: aCodeGenerator changeManager.
    codeGenerator compositeChangeCollector: aCodeGenerator compositeChangeCollector.
    nestingCount := aCodeGenerator compositeChangeNesting.
    nestingCount isNil ifTrue: [ 
        nestingCount := 0 
    ].  
    codeGenerator compositeChangeNesting: (1 + nestingCount).
    ^ codeGenerator

    "Created: / 19-04-2014 / 10:15:21 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 08-07-2014 / 16:57:32 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomCodeGenerator class methodsFor:'testing'!

isAbstract
    ^ self == CustomCodeGenerator

    "Created: / 26-01-2014 / 21:38:59 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!CustomCodeGenerator methodsFor:'accessing'!

builder

    ^ builder

    "Created: / 19-04-2014 / 10:16:28 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

builder: aBuilder

    builder := aBuilder

    "Created: / 19-04-2014 / 09:17:03 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

changeManager

    ^ changeManager

    "Created: / 31-05-2014 / 13:29:31 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

changeManager: aChangeManager

    changeManager := aChangeManager

    "Created: / 31-05-2014 / 13:30:02 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

compositeChangeCollector

    ^ compositeChangeCollector

    "Created: / 19-04-2014 / 10:18:06 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

compositeChangeCollector: aCompositeChangeCollector

    compositeChangeCollector := aCompositeChangeCollector

    "Created: / 19-04-2014 / 10:18:23 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

compositeChangeNesting

    ^ compositeChangeNesting

    "Created: / 11-05-2014 / 14:01:23 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

compositeChangeNesting: aNumber

    compositeChangeNesting := aNumber

    "Created: / 11-05-2014 / 14:01:56 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

dialog

    ^ dialog

    "Created: / 11-05-2014 / 00:27:21 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

dialog: aDialog

    dialog := aDialog

    "Created: / 11-05-2014 / 00:27:49 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

userPreferences

    ^ userPreferences

    "Created: / 09-06-2014 / 21:49:33 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

userPreferences: aUserPreferences

    userPreferences := aUserPreferences

    "Created: / 09-06-2014 / 21:49:56 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomCodeGenerator methodsFor:'bulk changes'!

executeCollectedChangesNamed:name
    "
    Same as CodeGeneratorTool >> executeCollectedChangesNamed:,
    but with custom RefactoryChangeManager and custom Dialog
    "

    compositeChangeCollector notNil ifTrue:[
        compositeChangeNesting := compositeChangeNesting - 1.
        compositeChangeNesting == 0 ifTrue:[
            compositeChangeCollector name:name.
            compositeChangeCollector changesSize == 0 ifTrue:[
                dialog information: 'Nothing generated.'.
            ] ifFalse:[
                changeManager performChange: compositeChangeCollector
            ].
            compositeChangeCollector := nil.
        ]
    ]

    "Created: / 31-05-2014 / 11:30:11 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomCodeGenerator methodsFor:'code generation'!

addBuilderChanges
    | change |

    change := builder change.
    change notNil ifTrue: [
        self addChange: change 
    ]

    "Created: / 10-04-2014 / 23:26:12 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 13-05-2014 / 22:10:40 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

createClass: definitionString
    "Create a new class defined by `definitionString`. 
    `definitionString` is something like:

    CustomCodeGeneratorOrRefactoring subclass:#CustomCodeGenerator
        instanceVariableNames:'sourcesInBuildProcess'
        classVariableNames:''
        poolDictionaries:''
        category:'Interface-Refactoring-Custom'     

    "


    | classChange  |

    classChange := InteractiveAddClassChange definition: definitionString.
    self addChange: classChange.

    "Created: / 03-04-2014 / 10:27:51 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!CustomCodeGenerator methodsFor:'executing'!

buildInContext: aCustomContext

    self subclassResponsibility

    "Created: / 18-03-2014 / 22:52:09 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

executeChangesInContext: aCustomContext changesName: aName

    self executeCollectedChangesNamed: aName.

    "Created: / 10-04-2014 / 23:26:56 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

executeInContext: aCustomContext
    | wg executor |

    wg := WindowGroup activeGroup.
    wg notNil ifTrue:[
        executor := [:whatToDo | wg withWaitCursorDo: [ whatToDo value ] ]
    ] ifFalse:[
        executor := [:whatToDo | whatToDo value ]
    ].

    executor value:[
        self startCollectChanges.       
        self buildInContext: aCustomContext.
        self addBuilderChanges.
        self executeChangesInContext: aCustomContext changesName: ('Add ', self class description).
    ]

    "Created: / 19-03-2014 / 18:45:26 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 03-04-2014 / 11:04:44 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 26-04-2014 / 16:15:19 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

executeSubGeneratorClasses: aSubGeneratorClasses inContext: aCustomContext

     aSubGeneratorClasses do: [ :aSubGeneratorClass | 
         | subGenerator |

         subGenerator := aSubGeneratorClass subGeneratorOf: self.
         subGenerator executeInContext: aCustomContext  
     ]

    "Created: / 08-07-2014 / 18:31:10 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomCodeGenerator methodsFor:'initialization'!

initialize

    super initialize.
    self setUpBuilder.
    self setUpDialog.
    self setUpChangeManager

    "Created: / 17-03-2014 / 22:27:32 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 09-06-2014 / 22:58:10 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

setUpBuilder

    builder := CustomSourceCodeBuilder new.

    "Created: / 09-06-2014 / 22:56:10 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

setUpChangeManager

    changeManager := CustomBrowserChangeManager new.

    "Created: / 09-06-2014 / 22:56:56 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

setUpDialog

    dialog := CustomUserDialog new.

    "Created: / 09-06-2014 / 22:57:08 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomCodeGenerator methodsFor:'private'!

addSource: source

    source sourceExists ifFalse: [
        self addChange: (source change).
    ]

    "Created: / 01-04-2014 / 21:53:54 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 10-04-2014 / 23:07:53 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomCodeGenerator class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

