"{ Package: 'jn:refactoring_custom' }"

CodeGeneratorTool subclass:#CustomCodeGeneratorOrRefactoring
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Interface-Refactoring-Custom'
!


!CustomCodeGeneratorOrRefactoring class methodsFor:'accessing-presentation'!

description
    "Returns more detailed description of the receiver"

    ^ self subclassResponsibility

    "Created: / 01-12-2013 / 00:18:41 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

group
    "Returns a collection strings describing a group to which
     receiver belongs. A groups may be nested hence the array of
     strings. For example for subgroup 'accessors' in group 'generate'
     this method should return #('generate' 'accessors')."

    "/ By default return an empty array which means the item will appear
    "/ in top-level group.
    ^ #()

    "Created: / 01-12-2013 / 00:21:48 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

label
    "Returns show label describing the receiver. This label
     is used in UI as menu item/tree item label."

    ^ self subclassResponsibility

    "Created: / 01-12-2013 / 00:18:05 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!CustomCodeGeneratorOrRefactoring class methodsFor:'executing'!

executeInContext: aCustomContext
    ^ self new executeInContext: aCustomContext

    "Created: / 26-01-2014 / 13:42:32 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!CustomCodeGeneratorOrRefactoring class methodsFor:'queries'!

availableInContext: aCustomContext
    "Returns true if the generator/refactoring is available in given
     context, false otherwise. 

     Called by the UI to figure out what generators / refactorings 
     are available at given point. See class CustomContext for details."

    ^ self subclassResponsibility

    "Created: / 01-12-2013 / 00:13:28 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

availableInPerspective:aCustomPerspective 
    "Returns true if the generator/refactoring is available in given
     perspective, false otherwise.

     Called by the UI to figure out what generators / refactorings
     to show"
    
    ^ self subclassResponsibility

    "Created: / 26-01-2014 / 13:03:25 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!CustomCodeGeneratorOrRefactoring class methodsFor:'testing'!

isAbstract
    ^ self == CustomCodeGeneratorOrRefactoring

    "Created: / 26-01-2014 / 21:38:30 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!CustomCodeGeneratorOrRefactoring methodsFor:'compilation'!

compile:theCode forClass:classOrClassName inCategory:categoryOrNil skipIfSame:skipIfSame 
    "install some code for a class.
     If refactory browser stuff is avaliable the refactory tools are used to support undo"

    |change compiler selector oldMethod isSame category class |

    isSame := false.
    class := classOrClassName isString 
                ifTrue:[ Smalltalk classNamed: classOrClassName asSymbol ]
                ifFalse:[ classOrClassName ].

    category := categoryOrNil ? (Compiler defaultMethodCategory).

    skipIfSame ifTrue:[
        compiler := class notNil 
                        ifTrue:[ class compilerClass new ] 
                        ifFalse:[ Compiler new "Defaults to Smalltalk compiler" ].

        compiler parseMethod:theCode in:class ignoreErrors:true ignoreWarnings:true.

        selector := compiler selector.
        (selector notNil and:[class notNil]) ifTrue:[
            oldMethod := class compiledMethodAt:selector.
            isSame := (oldMethod notNil and:[oldMethod source = theCode]).
            isSame ifTrue:[^ self ].
            oldMethod notNil ifTrue:[
                category := categoryOrNil ? (oldMethod category).
            ].
        ].
    ].

    self canUseRefactoringSupport ifFalse:[
        "/ compile immediately
        class isNil ifTrue:[ 
            self error: ('Class %1 does not exist' bindWith: classOrClassName)
        ].
        class compile:theCode classified:category.
        ^ self.
    ].

    "/ At this point we need a kind of a class. If there's no real class
    "/ yet, create a RBClass...
    class isNil ifTrue:[ 
        class := RBClass named: classOrClassName asSymbol.
    ].
    change := InteractiveAddMethodChange compile:(theCode asString) in:class classified:category.

    "/ if collecting, add to changes (to be executed as one change at the end,
    "/ in order to have only one change in the undo-list (instead of many)
    compositeChangeCollector notNil ifTrue:[
        compositeChangeCollector addChange:change
    ] ifFalse:[
        RefactoryChangeManager performChange:change.
    ]

    "Created: / 03-04-2014 / 10:45:20 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!CustomCodeGeneratorOrRefactoring methodsFor:'executing'!

buildInContext: aCustomContext

    self subclassResponsibility

    "Created: / 18-03-2014 / 22:50:30 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

executeInContext: aCustomContext
    "Generates the code or perform the refactoring. This method
     is guaranteed to be called only if #availableInContext: returns
     true. 

     Called by the UI when user want to perform the task"

    self subclassResponsibility

    "Created: / 01-12-2013 / 00:15:49 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!CustomCodeGeneratorOrRefactoring class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

