"{ Package: 'jn:refactoring_custom' }"

CustomCodeGenerator subclass:#CustomFooBarBazCodeGenerator
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Interface-Refactoring-Custom'
!


!CustomFooBarBazCodeGenerator class methodsFor:'accessing-presentation'!

description
    "Returns more detailed description of the receiver"

    ^ 'Generates three bogus methods - foo, bar and baz'

    "Modified: / 26-01-2014 / 21:46:09 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

label
    "Returns show label describing the receiver. This label
     is used in UI as menu item/tree item label."

    ^ 'foo, bar, baz...'

    "Modified: / 26-01-2014 / 21:45:50 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!CustomFooBarBazCodeGenerator class methodsFor:'queries'!

availableInContext: aCustomContext
    "Returns true if the generator/refactoring is available in given
     context, false otherwise. 

     Called by the UI to figure out what generators / refactorings 
     are available at given point. See class CustomContext for details."

    ^ aCustomContext selectedClasses notEmptyOrNil

    "Created: / 26-01-2014 / 21:41:56 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 26-01-2014 / 23:39:18 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 04-03-2014 / 22:34:19 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

availableInPerspective:aCustomPerspective 
    "Returns true if the generator/refactoring is available in given
     perspective, false otherwise.

     Called by the UI to figure out what generators / refactorings
     to show"

    ^ aCustomPerspective isClassPerspective

    "Created: / 26-01-2014 / 21:40:16 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 04-03-2014 / 21:41:34 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomFooBarBazCodeGenerator methodsFor:'executing'!

executeInContext:aCustomContext
    "Generates the code or perform the refactoring. This method
     is guaranteed to be called only if #availableInContext: returns
     true.
     
     Called by the UI when user want to perform the task"

    | classes |

    classes := aCustomContext selectedClasses.
    classes do:[:class | 
        (class canUnderstand: #foo) ifFalse:[  
            self compile: 'foo
    ^ self bar' forClass: class inCategory: 'foo - bar - baz'
        ].
        (class canUnderstand: #bar) ifFalse:[  
            self compile: 'bar
    ^ self baz' forClass: class inCategory: 'foo - bar - baz'
        ].
        (class canUnderstand: #baz) ifFalse:[  
            self compile: 'baz
    ^ 42' forClass: class inCategory: 'foo - bar - baz'
        ].

    ].

    "Modified: / 26-01-2014 / 23:41:48 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!CustomFooBarBazCodeGenerator class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

