"{ Package: 'jn:refactoring_custom' }"

CustomCodeGenerator subclass:#CustomIsAbstractCodeGenerator
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Interface-Refactoring-Custom'
!

!CustomIsAbstractCodeGenerator class methodsFor:'accessing-presentation'!

description
    "Returns more detailed description of the receiver"

    ^ 'Mark class as abstract (by implementing class method #isAbstract)'

    "Created: / 20-03-2014 / 01:31:33 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

label
    "Returns show label describing the receiver. This label
     is used in UI as menu item/tree item label."

    ^ 'Mark class as abstract'

    "Created: / 20-03-2014 / 01:32:28 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!CustomIsAbstractCodeGenerator class methodsFor:'queries'!

availableInContext:aCustomContext
    "Returns true if the generator/refactoring is available in given
     context, false otherwise.
     
     Called by the UI to figure out what generators / refactorings
     are available at given point. See class CustomContext for details."

    ^ aCustomContext selectedClasses notEmptyOrNil

    "Created: / 20-03-2014 / 01:34:11 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

availableInPerspective:aCustomPerspective
    "Returns true if the generator/refactoring is available in given
     perspective, false otherwise.
     
     Called by the UI to figure out what generators / refactorings
     to show"

    ^ aCustomPerspective isClassPerspective

    "Created: / 20-03-2014 / 01:33:48 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!CustomIsAbstractCodeGenerator methodsFor:'executing'!

executeInContext:aCustomContext
    "Generates the code or perform the refactoring. This method
     is guaranteed to be called only if #availableInContext: returns
     true.
     
     Called by the UI when user want to perform the task a test"

    | classes |

    classes := aCustomContext selectedClasses.
    classes do:[:class | 
        (class canUnderstand: #isAbstract) ifFalse:[  
            self compile: ('isAbstract
    ^ self == %1' bindWith: class) forClass: class class inCategory: 'testing'
        ].   
    ]

    "Created: / 20-03-2014 / 01:38:47 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified (comment): / 31-03-2014 / 23:26:25 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

