"{ Package: 'jn:refactoring_custom' }"

TestCase subclass:#CustomMockTests
	instanceVariableNames:'builder mock'
	classVariableNames:''
	poolDictionaries:''
	category:'Interface-Refactoring-Custom-Tests-Tests'
!

!CustomMockTests methodsFor:'initialization & release'!

setUp

    builder := CustomSourceCodeBuilder new.
    mock := CustomMock new

    "Modified (format): / 15-06-2014 / 19:12:14 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

tearDown

    builder redoChanges.
    mock unmockAll

    "Modified: / 11-07-2014 / 13:20:42 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomMockTests methodsFor:'tests'!

test_class_and_instance_methods_overriden_by_mock_block_value
    | class mockClass |

    class := builder createClassImmediate: 'TestClassForMockTestCase' superClassName: 'Object'.
    builder createMethodImmediate: (class theMetaclass) protocol: 'p' source: 'aSelector_01: aParam
    ^ 10'.
    builder createMethodImmediate: class protocol: 'p' source: 'aSelector_01: aParam
    ^ 15'.

    self assert: (class aSelector_01: nil) = 10.
    self assert: (class new aSelector_01: nil) = 15.

    mockClass := mock mockClassOf: class.
    mockClass mockSelector: #aSelector_01: withBlockValue: [ :aParam | ^ 20 ].
    mockClass new mockSelector: #aSelector_01: withBlockValue: [ :aParam | ^ 25 ].

    self assert: (mockClass aSelector_01: nil) = 20.
    self assert: (mockClass new aSelector_01: nil) = 25.

    "Created: / 10-07-2014 / 19:06:47 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_class_methods_overriden_by_mock
    | class mockClass |

    class := builder createClassImmediate: 'TestClassForMockTestCase' superClassName: 'Object'.
    builder createMethodImmediate: class theMetaclass protocol: 'p' source: 'aSelector_01: aParam
    ^ 10'.
    builder createMethodImmediate: class theMetaclass protocol: 'p' source: 'aSelector_02: aParam

    (self aSelector_01: aParam) = 10 ifTrue: [ ^ true ].

    ^ false'.

    self assert: (class aSelector_02: nil).

    mockClass := mock mockClassOf: class.
    mockClass mockSelector: #aSelector_01: withReturnValue: 20.

    self assert: (mockClass aSelector_01: nil) = 20.
    self assert: (mockClass aSelector_02: nil) not.

    "Created: / 10-07-2014 / 19:35:52 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_class_methods_overriden_by_mock_block_value
    | class mockClass |

    class := builder createClassImmediate: 'TestClassForMockTestCase' superClassName: 'Object'.
    builder createMethodImmediate: (class theMetaclass) protocol: 'p' source: 'aSelector_01: aParam
    ^ 10'.
    builder createMethodImmediate: (class theMetaclass) protocol: 'p' source: 'aSelector_02: aParam

    (self aSelector_01: aParam) = 10 ifTrue: [ ^ true ].

    ^ false'.

    self assert: (class aSelector_02: nil).

    mockClass := mock mockClassOf: class.
    mockClass mockSelector: #aSelector_01: withBlockValue: [ :aParam | ^ 20 ].

    self assert: (mockClass aSelector_01: nil) = 20.
    self assert: (mockClass aSelector_02: nil) not.

    "Created: / 10-07-2014 / 18:21:33 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_methods_overriden_by_mock
    | class mockClassInstance |

    class := builder createClassImmediate: 'TestClassForMockTestCase' superClassName: 'Object'.
    builder createMethodImmediate: class protocol: 'p' source: 'aSelector_01: aParam
    ^ 10'.
    builder createMethodImmediate: class protocol: 'p' source: 'aSelector_02: aParam

    (self aSelector_01: aParam) = 10 ifTrue: [ ^ true ].

    ^ false'.

    self assert: (class new aSelector_02: nil).

    mockClassInstance := mock mockOf: class.
    mockClassInstance mockSelector: #aSelector_01: withReturnValue: 20.

    self assert: (mockClassInstance aSelector_01: nil) = 20.
    self assert: (mockClassInstance aSelector_02: nil) not.

    "Created: / 15-06-2014 / 20:19:47 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 15-06-2014 / 23:56:56 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_methods_overriden_by_mock_block_value
    | class mockClassInstance |

    class := builder createClassImmediate: 'TestClassForMockTestCase' superClassName: 'Object'.
    builder createMethodImmediate: class protocol: 'p' source: 'aSelector_01: aParam
    ^ 10'.
    builder createMethodImmediate: class protocol: 'p' source: 'aSelector_02: aParam

    (self aSelector_01: aParam) = 10 ifTrue: [ ^ true ].

    ^ false'.

    self assert: (class new aSelector_02: nil).

    mockClassInstance := mock mockOf: class.
    mockClassInstance mockSelector: #aSelector_01: withBlockValue: [ :aParam | ^ 20 ].

    self assert: (mockClassInstance aSelector_01: nil) = 20.
    self assert: (mockClassInstance aSelector_02: nil) not.

    "Created: / 17-06-2014 / 21:48:00 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 19-06-2014 / 22:47:16 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_methods_overriden_by_mock_block_value_with_none_params
    | class mockClassInstance |

    class := builder createClassImmediate: 'TestClassForMockTestCase' superClassName: 'Object'.
    builder createMethodImmediate: class protocol: 'p' source: 'aSelector_01
    ^ 10'.

    self assert: (class new aSelector_01 = 10).

    mockClassInstance := mock mockOf: class.
    mockClassInstance mockSelector: #aSelector_01 withBlockValue: [ ^ 30 ].

    self assert: (mockClassInstance aSelector_01) = 30

    "Created: / 20-06-2014 / 09:02:04 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_methods_overriden_by_mock_block_value_with_params
    | class mockClassInstance |

    class := builder createClassImmediate: 'TestClassForMockTestCase' superClassName: 'Object'.
    builder createMethodImmediate: class protocol: 'p' source: 'aSelector_01: aParam
    ^ 10'.
    builder createMethodImmediate: class protocol: 'p' source: 'aSelector_02: aParam

    (self aSelector_01: aParam) = 10 ifTrue: [ ^ true ].

    ^ false'.

    self assert: (class new aSelector_02: nil).

    mockClassInstance := mock mockOf: class.
    mockClassInstance mockSelector: #aSelector_01: withBlockValue: [ :aParam | 
        aParam = 5 ifTrue: [
            ^ 10
        ].

        ^ 30
    ].

    self assert: (mockClassInstance aSelector_01: 5) = 10.
    self assert: (mockClassInstance aSelector_02: 3) not.

    "Created: / 17-06-2014 / 21:56:37 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_methods_overriden_by_mock_block_value_with_three_params
    | class mockClassInstance |

    class := builder createClassImmediate: 'TestClassForMockTestCase' superClassName: 'Object'.
    builder createMethodImmediate: class protocol: 'p' source: 'aSelector_01: arg_01 param_02: arg_02 param_03: arg_03
    ^ 10'.

    self assert: ((class new aSelector_01: 1 param_02: 2 param_03: 3) = 10).

    mockClassInstance := mock mockOf: class.
    mockClassInstance mockSelector: #aSelector_01:param_02:param_03: withBlockValue: [ 
        :arg_01 :arg_02 :arg_03 | 

        ^ arg_01 + arg_02 + arg_03
    ].

    self assert: (mockClassInstance aSelector_01: 3 param_02: 3 param_03: 3) = 9

    "Created: / 20-06-2014 / 09:22:54 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_methods_overriden_by_mock_block_value_with_two_params
    | class mockClassInstance |

    class := builder createClassImmediate: 'TestClassForMockTestCase' superClassName: 'Object'.
    builder createMethodImmediate: class protocol: 'p' source: 'aSelector_01: arg_01 param_02: arg_02
    ^ 10'.

    self assert: ((class new aSelector_01: 1 param_02: 2) = 10).

    mockClassInstance := mock mockOf: class.
    mockClassInstance mockSelector: #aSelector_01:param_02: withBlockValue: [ :arg_01 :arg_02 | ^ arg_01 + arg_02 ].

    self assert: (mockClassInstance aSelector_01: 3 param_02: 3) = 6

    "Created: / 20-06-2014 / 09:19:53 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

