"{ Package: 'jn:refactoring_custom' }"

CodeGenerator subclass:#CustomSourceCodeBuilder
	instanceVariableNames:'codeBuilds changeManager defaultValues defaultReplacements'
	classVariableNames:''
	poolDictionaries:''
	category:'Interface-Refactoring-Custom'
!

CustomSourceCodeBuilder subclass:#Class
	instanceVariableNames:'className isMeta superclassName instanceVariableNames
		classVariableNames poolDictionaryNames category'
	classVariableNames:''
	poolDictionaries:''
	privateIn:CustomSourceCodeBuilder
!

CustomSourceCodeBuilder subclass:#Method
	instanceVariableNames:'classes commentPlaceholderMarker commentReplacements'
	classVariableNames:''
	poolDictionaries:''
	privateIn:CustomSourceCodeBuilder
!


!CustomSourceCodeBuilder class methodsFor:'instance creation'!

classBuilder

    ^ CustomSourceCodeBuilder::Class new

    "Created: / 31-03-2014 / 18:05:05 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

methodBuilder

    ^ CustomSourceCodeBuilder::Method new

    "Created: / 09-04-2014 / 23:47:59 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomSourceCodeBuilder methodsFor:'accessing'!

buildedSource

    self subclassResponsibility

    "Created: / 18-05-2014 / 16:48:53 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

change

   self createChange.
   ^change

    "Created: / 24-03-2014 / 09:02:36 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomSourceCodeBuilder methodsFor:'code creation'!

createClass
    "Much like createMethod, but for class"
    | classBuilder |

    classBuilder := self class classBuilder.
    self fillDefaultValuesTo: classBuilder.  
    codeBuilds add: classBuilder.  
    ^ classBuilder

    "Created: / 09-04-2014 / 21:38:20 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 29-04-2014 / 20:51:35 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified (comment): / 18-05-2014 / 11:59:00 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

createClassImmediate: aClassName superClassName: aSuperClassName
    "Creates class immediately and returns the real class"

    (self createClass)
        className: aClassName;
        superclassName: aSuperClassName.

    self execute.

    ^ Smalltalk classNamed: aClassName

    "Created: / 15-06-2014 / 15:59:21 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

createMethod
    "
    Creates, returns method builder with adding it to codeBuils
    so code changes can be created withing this class at once -
    see method createChange.
    "
    | methodBuilder |

    methodBuilder := self methodBuilder.
    codeBuilds add: methodBuilder.  
    ^ methodBuilder

    "Created: / 09-04-2014 / 23:54:03 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified (comment): / 18-05-2014 / 11:57:59 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

createMethodImmediate: aClass protocol: aProtocol source: aSource
    "Much like createClassImmediate:superClassName:, but for method"

    (self createMethod)
        class: aClass;
        protocol: aProtocol;
        source: aSource.

    self execute

    "Created: / 15-06-2014 / 16:06:00 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomSourceCodeBuilder methodsFor:'compiling'!

execute
    "
    Most likely as compile method, but without undo change
    in global RefactoryChangeManager
    (true for CustomLocalChangeManager changeManager implementation)
    "

    changeManager performChange: self change

    "Created: / 27-04-2014 / 16:30:05 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified (comment): / 31-05-2014 / 19:45:47 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

redoChanges
    "redo all changes made by execute method"

    changeManager redoChanges

    "Created: / 27-04-2014 / 17:07:52 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 31-05-2014 / 19:47:02 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomSourceCodeBuilder methodsFor:'default values'!

category:aString

    self storeDefaultValue: #category: value: aString

    "Created: / 29-04-2014 / 19:51:45 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

class:aClass

    self storeDefaultValue: #class: value: aClass

    "Created: / 29-04-2014 / 19:55:50 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

className:something

    self storeDefaultValue: #className: value: something

    "Created: / 29-04-2014 / 19:42:23 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

classVariableNames:something

    self storeDefaultValue: #classVariableNames: value: something

    "Created: / 29-04-2014 / 19:43:22 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

classes: aClassCollection

    self storeDefaultValue: #classes: value: aClassCollection

    "Created: / 29-04-2014 / 19:50:07 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

fillDefaultValuesTo: anObject

    defaultValues keysAndValuesDo: [ :selector :value |
        (anObject class includesSelector: selector) ifTrue: [
            anObject perform: selector with: value 
        ]    
    ].

    defaultReplacements keysAndValuesDo: [ :placeholder :code |
        anObject replace: placeholder with: code 
    ].

    "Created: / 29-04-2014 / 20:21:57 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 04-05-2014 / 11:50:20 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

forgetReplacements

    defaultReplacements removeAll

    "Created: / 29-04-2014 / 20:19:15 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

instanceVariableNames:something

    self storeDefaultValue: #instanceVariableNames: value: something

    "Created: / 29-04-2014 / 19:44:37 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

isMeta:aBoolean

    self storeDefaultValue: #isMeta: value: aBoolean

    "Created: / 29-04-2014 / 19:47:47 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

poolDictionaryNames:something

    self storeDefaultValue: #poolDictionaryNames: value: something

    "Created: / 29-04-2014 / 19:48:37 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

protocol:aString

    self storeDefaultValue: #protocol: value: aString

    "Created: / 29-04-2014 / 20:01:08 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

replace: placeholder with: code

    ((self class) == CustomSourceCodeBuilder) ifTrue: [
        defaultReplacements 
            at: placeholder 
            put: code
    ]
    ifFalse: [
        super replace: placeholder with: code   
    ]

    "Created: / 29-04-2014 / 20:02:45 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

source:aString

    self storeDefaultValue: #source: value: aString

    "Created: / 29-04-2014 / 20:12:20 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

storeDefaultValue: aSelector value: aValue

    ((self class) == CustomSourceCodeBuilder) ifTrue: [
        defaultValues 
            at: aSelector 
            put: aValue    
    ]
    ifFalse: [
        self error: 'Selector ', aSelector, ' has to be overwritten by class: ', self className.
        "
        | superclass |

        superclass := self class superclass.
        superclass superclass isNil ifFalse: [
            superclass := superclass superclass
        ].

        (superclass canUnderstand: aSelector) ifTrue: [
            self assert: (superclass == CustomSourceCodeBuilder) not.
            self perform: aSelector with: aValue   
        ]
        "
    ]

    "Created: / 28-04-2014 / 19:08:17 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 02-05-2014 / 22:45:36 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

superclassName:something

    self storeDefaultValue: #superclassName: value: something

    "Created: / 29-04-2014 / 19:49:02 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomSourceCodeBuilder methodsFor:'initialization'!

initialize
    "Invoked when a new instance is created."

    super initialize.
    codeBuilds := OrderedCollection new.
    changeManager := CustomLocalChangeManager new.
    defaultValues := Dictionary new.
    defaultReplacements := Dictionary new.

    "Created: / 09-04-2014 / 23:44:04 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 31-05-2014 / 19:42:56 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomSourceCodeBuilder methodsFor:'instance creation'!

methodBuilder
    "Creates and returns method source code builder with default values"

    | methodBuilder |

    methodBuilder := self class methodBuilder.
    self fillDefaultValuesTo: methodBuilder.   
    ^ methodBuilder

    "Created: / 18-05-2014 / 11:41:10 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomSourceCodeBuilder methodsFor:'private'!

createChange

    (codeBuilds length > 0) ifTrue: [
        change := CompositeRefactoryChange new.
        codeBuilds do: [ :builder | 
            change addChange: (builder change)
        ]
    ] ifFalse: [
        change := nil.
    ]

    "Created: / 10-04-2014 / 00:14:39 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomSourceCodeBuilder methodsFor:'queries'!

sourceExists
    "Tells whether source is already present in the target class, package ..."

    self subclassResponsibility

    "Created: / 23-03-2014 / 23:02:08 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomSourceCodeBuilder::Class methodsFor:'accessing'!

category:something
    category := something.

    "Created: / 04-05-2014 / 21:18:07 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

className:something
    className := something.
!

classNameBuilded
    ^ className

    "Created: / 14-04-2014 / 14:14:57 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

classVariableNames:something
    classVariableNames := something.
!

instVarNames

    ^ (instanceVariableNames asCollectionOfWords collect:[:varName| varName asSymbol]) asArray.
"    instanceVariableNames isNil ifTrue:[
        ^ #()
    ].

    instanceVariableNames isString ifTrue:[
        instanceVariableNames := (instanceVariableNames asCollectionOfWords collect:[:varName| varName asSymbol]) asArray.
    ].

    ^ instanceVariableNames"

    "Created: / 29-05-2014 / 23:46:45 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

instanceVariableNames:something
    instanceVariableNames := something.
!

isMeta
    ^ isMeta

    "Created: / 23-06-2014 / 19:22:44 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

isMeta:aBoolean
    isMeta := aBoolean.
!

poolDictionaryNames:something
    poolDictionaryNames := something.
!

superclassName:something
    superclassName := something.
!

theMetaclass

    isMeta ifTrue: [
        ^ self 
    ].

    ^ (self deepCopy) 
        isMeta: true;
        yourself

    "Created: / 26-04-2014 / 17:10:52 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

theNonMetaclass

    isMeta ifTrue: [
        ^ (self deepCopy) 
            isMeta: false;
            yourself
    ].  

    ^ self

    "Created: / 26-04-2014 / 17:11:30 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomSourceCodeBuilder::Class methodsFor:'class method stubs'!

allClassVarNames
    | variableNames |

    variableNames := self asRBClass allClassVariableNames.

    variableNames isNil ifTrue: [ 
        ^ #()
    ].

    ^ variableNames

    "Created: / 01-06-2014 / 23:40:50 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 02-06-2014 / 08:31:19 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

allSuperclassesDo: aBlock

    self superclass withAllSuperclassesDo: aBlock

    "Created: / 21-04-2014 / 19:15:49 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 21-04-2014 / 21:35:17 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

methodDictionary

    ^ MethodDictionary new

    "Created: / 21-04-2014 / 19:16:17 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

superclass
    | superclass |

    superclass := Smalltalk classNamed: superclassName.

    isMeta ifTrue: [
        ^ superclass class
    ].  

    ^ superclass

    "Created: / 21-04-2014 / 20:23:21 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 21-04-2014 / 21:34:59 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomSourceCodeBuilder::Class methodsFor:'conversion'!

asRBClass
    "Creates instance of RBClass for usage in Refactory Browser package "
    | namespace |

    namespace := RBNamespace new.
    namespace defineClass: self definitionString. 

    isMeta ifTrue: [
        ^ namespace metaclassNamed: className asSymbol.
    ] ifFalse: [
        ^ namespace classNamed: className asSymbol        
    ]

    "Created: / 21-04-2014 / 21:17:08 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 23-06-2014 / 18:59:31 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomSourceCodeBuilder::Class methodsFor:'initialization'!

initialize

    className := ''.
    isMeta := false.
    superclassName := 'Object'.
    instanceVariableNames := ''.
    classVariableNames := ''.
    poolDictionaryNames := ''.
    category := ''.

    "Created: / 30-03-2014 / 22:34:15 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomSourceCodeBuilder::Class methodsFor:'message delegation'!

doesNotUnderstand: aMessage
    "Delegate messages (when implemented) to RBClass or RBMetaclass
    depending on isMeta attribute. (To not duplicate implementation in this class)"

    | rbClass selector |

    rbClass := self asRBClass.
    selector := aMessage selector.

    (rbClass class canUnderstand: selector) ifTrue: [ 
        ^ aMessage sendTo: rbClass
    ].

    ^ super doesNotUnderstand: aMessage

    "Created: / 01-06-2014 / 20:36:22 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 23-06-2014 / 18:55:31 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomSourceCodeBuilder::Class methodsFor:'private'!

createChange

    change := InteractiveAddClassChange definition: self definitionString.

    "Created: / 30-03-2014 / 19:19:38 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 18-04-2014 / 21:57:41 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

definitionString

    ^ (superclassName, ' subclass:#', className, '
        instanceVariableNames:''', instanceVariableNames, '''
        classVariableNames:''', classVariableNames, '''
        poolDictionaries:''', poolDictionaryNames, '''
        category:''', category, '''').

    "Created: / 18-04-2014 / 21:57:22 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomSourceCodeBuilder::Class methodsFor:'queries'!

sourceExists

    ^ Smalltalk hasClassNamed: className

    "Created: / 29-03-2014 / 21:21:59 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 01-04-2014 / 21:20:25 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomSourceCodeBuilder::Method methodsFor:'accessing'!

buildedSource
    "
    Returns builded and formatted method source
    code as string (with replacements and so on)
    "

    | parser method sourceCode |

    sourceCode := self replaceCommentsInSource: source.

    parser := RBParser new.
    parser errorBlock:[ :str :pos | self error: ('Error: %1: %2' bindWith: pos with: str). ^ self ].
    parser initializeParserWith: sourceCode type: #rewriteSavingCommentsOn:errorBlock:.
    method := parser parseMethod: sourceCode.    

    "/ Transcript showCR: 'SRC1::: ', method source asString.
    method source: nil.
    method acceptVisitor: self.

    "/ Transcript showCR: 'SRC2::: ', method source asString.
    "/ Transcript showCR: 'BODY::: ', method body asString.

    ^ method formattedCode

    "Created: / 18-05-2014 / 16:48:27 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 03-07-2014 / 15:38:52 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

category:aString

    self protocol: aString

    "Created: / 02-05-2014 / 22:49:19 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

class:aClass

    self classes: (Array with: aClass)

    "Created: / 02-05-2014 / 22:49:42 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

classes: aClassCollection

    classes := aClassCollection

    "Created: / 24-04-2014 / 23:50:51 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

commentPlaceholderMarker: aString
    "
    Sets prefix string which will mark comment replace 
    in code replacements given by:
    replace: '`comment' with: 'comment'
    "

    commentPlaceholderMarker := aString

    "Created: / 25-05-2014 / 17:41:20 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

protocol:aString

    protocol := aString

    "Created: / 02-05-2014 / 22:53:31 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

replace: placeholder with: code

    (placeholder startsWith: commentPlaceholderMarker) ifTrue: [
        commentReplacements
            at: placeholder 
            put: code
    ]
    ifFalse: [
        replacements 
            at: placeholder
            put: (code isSymbol 
                    ifTrue:[code]
                    ifFalse:[RBParser parseRewriteExpression: code])
    ]

    "Created: / 02-05-2014 / 22:54:02 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 25-05-2014 / 20:17:13 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

source:aString

    source := aString

    "Created: / 02-05-2014 / 22:54:28 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomSourceCodeBuilder::Method methodsFor:'initialization'!

initialize
    "Invoked when a new instance is created."

    super initialize.
    commentPlaceholderMarker := '`"'.
    commentReplacements := Dictionary new.
    classes := #()

    "Created: / 25-05-2014 / 17:46:58 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 10-07-2014 / 14:28:16 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomSourceCodeBuilder::Method methodsFor:'private'!

createChange
    "Creates a refactory browser change (add method change)"

    change := nil.
    (classes length > 0) ifTrue: [
        change := CompositeRefactoryChange new.        
    ].

    classes do: [ :class |
        | methodClass methodChange |

        methodClass := class.
        (class isKindOf: CustomSourceCodeBuilder) ifTrue: [
            methodClass := class asRBClass
        ].  

        change addChange: (InteractiveAddMethodChange new
                                class: methodClass
                                protocol: protocol
                                source: self buildedSource)
    ]

    "Created: / 10-04-2014 / 00:03:34 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified (comment): / 18-05-2014 / 19:24:09 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

replaceCommentsInSource: aSourceString
    "
    Returns source string with replaced occurences of comment
    replaces given by:
    replace: '`{double_quote_char}comment' with: '{double_quote_char}a comment{double_quote_char}'
    where {double_quote_char} is "" (but not escaped like in this comment)
    "

    | sourceCode |

    sourceCode := source.

    commentReplacements keysAndValuesDo: [ :placeholder :code | 
        sourceCode := sourceCode copyReplaceString: placeholder withString: code       
    ].

    ^ sourceCode

    "Created: / 25-05-2014 / 16:55:56 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 25-05-2014 / 20:20:02 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomSourceCodeBuilder::Method methodsFor:'queries'!

sourceExists

    ^ (class includesSelector: (self change selector)).

    "Modified: / 24-03-2014 / 22:44:28 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomSourceCodeBuilder class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

