"{ Package: 'jn:refactoring_custom' }"

CustomCodeGeneratorTestCase subclass:#CustomSourceCodeBuilderTests
	instanceVariableNames:'refactoryChangeManager'
	classVariableNames:''
	poolDictionaries:''
	category:'Interface-Refactoring-Custom-Tests'
!

!CustomSourceCodeBuilderTests methodsFor:'accessing'!

generator

   ^ nil

    "Modified: / 31-05-2014 / 22:48:25 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomSourceCodeBuilderTests methodsFor:'initialization & release'!

setUp

    super setUp.
    refactoryChangeManager := RefactoryChangeManager instance.

    "Created: / 16-04-2014 / 21:39:50 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 31-05-2014 / 22:45:57 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomSourceCodeBuilderTests methodsFor:'private'!

lastUndoChange

    refactoryChangeManager undoableOperations isEmpty ifTrue: [ 
        ^ nil 
    ] ifFalse: [
        ^ refactoryChangeManager undoChange
    ].

    "Created: / 16-04-2014 / 21:40:37 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 31-05-2014 / 19:53:08 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomSourceCodeBuilderTests methodsFor:'tests'!

test_all_class_var_names
    | class actualClassVars expectedClassVars |

    class := builder createClass.
    class
        className: 'MockClassForTestCase';
        classVariableNames: 'ClassVar1 ClassVar2'.

    expectedClassVars := (Object allClassVarNames, (Array with: #ClassVar1 with: #ClassVar2)).
    actualClassVars := class theNonMetaclass allClassVarNames.

    self assert: expectedClassVars = actualClassVars

    "Created: / 20-06-2014 / 22:35:27 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 23-06-2014 / 19:09:33 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_chaining_api_create_method
    | class |

    class := self class.

    builder
        class: class;
        protocol: 'a protocol';
        source: '`@selector
            self shouldImplement';
        replace: '`@selector' with: 'aSelector';
        createMethod;

        replace: '`@selector' with: 'aSelector2';
        createMethod.

    self assert: (class includesSelector: #aSelector) not.
    self assert: (class includesSelector: #aSelector2) not.

    builder execute.

    self assert: (class includesSelector: #aSelector).
    self assert: (class includesSelector: #aSelector2).

    self assert: (class sourceCodeAt: #aSelector) = 'aSelector
    self shouldImplement'.

    self assert: (class sourceCodeAt: #aSelector2) = 'aSelector2
    self shouldImplement'.

    "Created: / 27-04-2014 / 17:48:30 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 28-04-2014 / 09:19:25 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_chaining_api_create_method_with_class
    | class className method |

    className := 'TestClassNameForTestCase'.

    class := builder
        className: className;
        classVariableNames: 'ClassVar1 ClassVar2';
        instanceVariableNames: 'instVar1 instVar2';
        poolDictionaryNames: 'dictName';
        superclassName: 'TestCase';
        category: 'Some-Category';
        createClass.

    builder 
        class: class;
        protocol: 'a protocol';
        source: '`@selector
            self shouldImplement';
        replace: '`@selector' with: 'aSelector';
        createMethod.

    self assertClassNotExists: className.

    builder execute.

    class := Smalltalk classNamed: className.

    self assert: class new className = className.
    self assert: class superclass new className = 'TestCase'.
    self assert: class instanceVariableString = 'instVar1 instVar2'.
    self assert: class classVariableString = 'ClassVar1 ClassVar2'.
    self assert: class poolDictionaries = 'dictName'.
    self assert: class category = 'Some-Category'.

    self assert: (class includesSelector: #aSelector).
    self assert: (class sourceCodeAt: #aSelector) = 'aSelector
    self shouldImplement'.
    method := class compiledMethodAt: #aSelector.  
    self assert: (method category) = 'a protocol'.

    "Created: / 04-05-2014 / 13:00:38 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 15-06-2014 / 17:16:02 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_class_and_class_method_generated_as_one_undo_change
    | className class method lastUndoChange |

    lastUndoChange := self lastUndoChange.

    className := 'MockClassForTestingOneUndoChange'.

    self assertClassNotExists: className.

    builder := CustomSourceCodeBuilder new.
    class := builder createClass.
    class className: className.  

    self assertClassNotExists: className.

    method := builder createMethod.
    method 
        class: class theMetaclass;
        protocol: 'a protocol';
        source: 'aSelector
        ^ nil'.

    self assertClassNotExists: className.
    self assert: lastUndoChange = (self lastUndoChange).

    builder compile.

    self assertClassExists: className.
    self assert: (lastUndoChange = (self lastUndoChange)) not.
    self assert: ((Smalltalk classNamed: className) class includesSelector: #aSelector).
    self assert: ((Smalltalk classNamed: className) includesSelector: #aSelector) not.

    refactoryChangeManager undoOperation.

    self assertClassNotExists: className.
    self assert: lastUndoChange = (self lastUndoChange).

    "Created: / 17-04-2014 / 23:54:22 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 15-06-2014 / 17:17:53 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_class_and_instance_method_generated_as_one_undo_change
    | className class method lastUndoChange |

    lastUndoChange := self lastUndoChange.

    className := 'MockClassForTestingOneUndoChange'.

    self assertClassNotExists: className.

    builder := CustomSourceCodeBuilder new.
    class := builder createClass.
    class className: className.  

    self assertClassNotExists: className.

    method := builder createMethod.
    method 
        class: class;
        protocol: 'a protocol';
        source: 'aSelector
        ^ nil'.

    self assertClassNotExists: className.
    self assert: lastUndoChange = (self lastUndoChange).

    builder compile.

    self assertClassExists: className.
    self assert: (lastUndoChange = (self lastUndoChange)) not.
    self assert: ((Smalltalk classNamed: className) includesSelector: #aSelector).

    refactoryChangeManager undoOperation.

    self assertClassNotExists: className.
    self assert: lastUndoChange = (self lastUndoChange).

    "Created: / 17-04-2014 / 23:54:05 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 15-06-2014 / 17:19:06 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_comment_in_method_generated
    | method class generatedSource expectedSource |

    class := self class.

    method := builder createMethod.
    method 
        class: self class;
        protocol: 'a protocol';
        source: 'aSelector
        "a comment"
        ^ nil'.

    self assert: (class includesSelector: #aSelector) not.

    builder execute.

    generatedSource := (class sourceCodeAt: #aSelector).
    expectedSource := 'aSelector
    "a comment"

    ^ nil'.


    self assert: (class includesSelector: #aSelector).
    self assert: (generatedSource includesSubString: '"a comment"').
    self assertSource: expectedSource sameAs: generatedSource

    "Created: / 27-04-2014 / 15:57:17 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 24-05-2014 / 00:34:31 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 25-05-2014 / 22:23:27 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_create_class_immediate
    | mockClass |

    mockClass := builder createClassImmediate: 'MockClassForTestCase' superClassName: 'Object'.

    self assert: mockClass new className = 'MockClassForTestCase'.
    self assert: mockClass superclass new className = 'Object'.
    self assert: mockClass instanceVariableString = ''.
    self assert: mockClass classVariableString = ''.
    self assert: mockClass poolDictionaries = ''.
    self assert: mockClass category = ''.

    "Created: / 15-06-2014 / 17:27:41 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_create_method_immediate
    | mockClass expectedSource actualSource |

    expectedSource := 'instanceMethod:aParam
    ^ self'.

    mockClass := builder createClassImmediate: 'MockClassForTestCase' superClassName: 'Object'.
    builder createMethodImmediate: mockClass protocol: 'a protocol' source: expectedSource.

    actualSource := (mockClass sourceCodeAt: #instanceMethod:).

    self assertSource: expectedSource sameAs: actualSource.
    self assert: (mockClass compiledMethodAt: #instanceMethod:) category = 'a protocol'

    "Created: / 15-06-2014 / 17:28:36 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 24-06-2014 / 21:59:23 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_method_source_builded
    | methodBuilder source |

    methodBuilder := builder methodBuilder.
    methodBuilder
        source: '`@methodName

        ^ `variableName';
        replace: '`@methodName' with: #selector;
        replace: '`variableName' with: 'aName'.

    source := methodBuilder buildedSource.

    self assert: source = 'selector
    ^ aName'.

    "Created: / 18-05-2014 / 17:14:13 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 19-05-2014 / 19:01:29 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 24-05-2014 / 00:20:11 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

test_method_source_builded_with_comment
    | methodBuilder buildedSource expectedSource |

    methodBuilder := builder methodBuilder.
    methodBuilder
        source: '`@methodName

        `"comment

        ^ `variableName';
        replace: '`@methodName' with: #selector;
        replace: '`variableName' with: 'aName';
        replace: '`"comment' with: '"a comment"'.

    buildedSource := methodBuilder buildedSource.
    expectedSource := 'selector
    "a comment"

    ^ aName'.

    self assertSource: expectedSource sameAs: buildedSource

    "Created: / 19-05-2014 / 18:57:25 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 25-05-2014 / 22:22:46 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_one_method_for_multiple_classes
    | classCollection classNames |

    classCollection := OrderedCollection new.
    classNames := Array with: 'DummyClassForTestCase1' with: 'DummyClassForTestCase2'.

    classNames do: [ :className |
        classCollection add: (builder className: className; createClass).
    ].

    builder
        classes: classCollection;
        protocol: 'a protocol';
        source: '`@selector
            self shouldImplement';
        replace: '`@selector' with: 'aSelector';
        createMethod.

    builder execute.

    classNames do: [ :className |
        | class method |

        class := Smalltalk classNamed: className.
        self assert: (class includesSelector: #aSelector).
        self assert: (class sourceCodeAt: #aSelector) = 'aSelector
    self shouldImplement'.
        method := class compiledMethodAt: #aSelector.  
        self assert: (method category) = 'a protocol'.
    ].

    "Created: / 02-05-2014 / 23:31:07 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 04-05-2014 / 11:51:57 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_redo_changes_for_multiple_executes
    "Note: Methods createClassImmediate and createMethodImmediate have execute call inside"
    | mockSuperClassName mockClassName mockSuperClass |

    mockSuperClassName := 'MockSuperClassForTestCase'.
    mockClassName := 'MockClassForTestCase'.

    mockSuperClass := builder createClassImmediate: mockSuperClassName superClassName: 'Object'.
    builder createClassImmediate: mockClassName superClassName: mockSuperClassName.

    "/ Instance method
    builder createMethodImmediate: mockSuperClass protocol: 'instance-protocol' source: 'instanceMethod: aParam
    self shouldImplement'.

    "/ Class method
    builder createMethodImmediate: mockSuperClass class protocol: 'class-protocol' source: 'classMethod: aParam
    self shouldImplement'.

    self assertClassExists: mockSuperClassName.
    self assertClassExists: mockClassName.

    builder redoChanges.

    self assertClassNotExists: mockSuperClassName.
    self assertClassNotExists: mockClassName.

    "Created: / 15-06-2014 / 16:21:52 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

